/* wmspeedfreq 1.0 
   (c) Tom Kistner 2004
   http://duncanthrax.net/wmspeedfreq/
   
   Loosely based on wmapm [1] by Mike Henderson.
   Uses the speedfreq library [2] by Jeremy Fitzhardinge.
   
  [1] http://nis-www.lanl.gov/~mgh/WindowMaker/DockApps.shtml
  [2] http://www.goop.org/~jeremy/speedfreq/
*/
 
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <X11/X.h>
#include <X11/xpm.h>
#include <speedfreq.h>
#include "wmspeedfreq.h"
#include "../wmgeneral/wmgeneral.h"
#include "wmspeedfreq_master.xpm"
#include "wmspeedfreq_mask.xbm"

void ParseCMDLine(int argc, char *argv[]);
void pressEvent(XButtonEvent *xev);
struct sf_cpuinfo cpuinfo;

int main(int argc, char *argv[]) {
  struct sf_mon_event *evt;
  int digit;
  XEvent event;
  
  if (sf_connect()) {
    /* connection to daemon failed */
    printf("Can't connect to speedfreqd daemon.\n");
    exit(255);
  };
  
  evt = malloc(sizeof(struct sf_mon_event));
  
  sf_getcpuinfo(&cpuinfo);
  
  openXwindow(argc, argv,
              wmspeedfreq_master,
              wmspeedfreq_mask_bits,
              wmspeedfreq_mask_width,
              wmspeedfreq_mask_height);

  while(1) {

    /* Process any pending X events */
    while(XPending(display)) {
      XNextEvent(display, &event);
      switch(event.type){
        case Expose:
          RedrawWindow();
          break;
        case ButtonPress:
          pressEvent(&event.xbutton);
          break;
        case ButtonRelease:
          break;
      }
    }
    RedrawWindow();
    
    if (sf_poll(evt, 0) > 0) {
      
      switch (evt->type) {
        case EVT_SHUTDOWN:
          printf("speedfreqd shutdown, exiting\n");
          exit(0);
        break;
      
        case EVT_CPUINFO:
          show_mhz(evt->u.cpuinfo.current);
        break;
        
        case EVT_POLICY:
          set_policy(&(evt->u.policy));
        break;
      }; /* switch */
    };

    usleep(DELAY);
  }
}


void set_policy(struct sf_policyinfo *policy) {
  
  switch (policy->policy) {
    case SF_POWERSAVE:
      /* use power saving policy button */
      copyXPMArea(5, 77, 25, 11, 5, 48);
      /* overpaint other buttons */
      copyXPMArea(63, 64, 54, 11, 5, 33);
      copyXPMArea(34, 64, 25, 11, 34, 48);
    break;
    case SF_PERFORMANCE:
      /* use performance policy button */
      copyXPMArea(34, 77, 25, 11, 34, 48);
      /* overpaint other buttons */
      copyXPMArea(63, 64, 54, 11, 5, 33);
      copyXPMArea(5, 64, 25, 11, 5, 48);
    break;
    case SF_DYNAMIC:
      /* use dynamic policy button */
      copyXPMArea(63, 77, 54, 11, 5, 33);
      /* overpaint other buttons */
      copyXPMArea(5, 64, 25, 11, 5, 48);
      copyXPMArea(34, 64, 25, 11, 34, 48);
    break;
    case SF_FIXED:
      /* overpaint other buttons */
      copyXPMArea(63, 64, 54, 11, 5, 33);
      copyXPMArea(5, 64, 25, 11, 5, 48);
      copyXPMArea(34, 64, 25, 11, 34, 48);
    break;
    
  }; /* switch */

  RedrawWindow();
  return;
}


void show_mhz(unsigned kHz) {
  int i;
  char buffer[5];
  memset(buffer,0,5);
  snprintf(buffer, 5, "%4d", (kHz / 1000));

  for (i=0;i<4;i++) {
    if (buffer[i] == ' ') {
      /* blank zero */
      copyXPMArea(75, 93, 6, 9, 7+(7*i), 7);
    }
    else {
      /* a standard digit */
      copyXPMArea(((buffer[i]-48)*7)+5, 93, 6, 9, 7+(7*i), 7);
    };
  };
  
  /* update speed bar */
  copyXPMArea(65, 18, 49, 9, 7, 18);
  copyXPMArea(65, 40, ((kHz-cpuinfo.min) / ((cpuinfo.max-cpuinfo.min) / 49)), 9, 7, 18);
  
  RedrawWindow();
  return;
}


void pressEvent(XButtonEvent *xev){

  int x=xev->x;
  int y=xev->y;

  if(x>=5 && y>=33 && x<=58 && y<=43) { 
    /* dynamic policy */
    sf_pol_dynamic(0,0);
  }
  else if (x>=5 && y>=48 && x<=29 && y<=58) {
    sf_pol_powersave();
  }
  else if (x>=34 && y>=48 && x<=58 && y<=58) {
    sf_pol_performance();
  }
  else if (x>=7 && y>=18 && x<=55 && y<=26) {
    /* we have 49 pixels */
    sf_pol_fixed(((x-7)*((cpuinfo.max-cpuinfo.min) / 49))+cpuinfo.min);
  }
  
  return;
}





